@@ -1,7 +1,7 @@
 
 /*
  *  autogen.c
- *  $Id: autogen.c,v 4.7 2005/04/16 16:44:27 bkorb Exp $
+ *  $Id: autogen.c,v 4.10 2005/12/05 20:51:36 bkorb Exp $
  *  This is the main routine for autogen.
  */
 
@@ -21,8 +21,8 @@
  *  You should have received a copy of the GNU General Public License
  *  along with AutoGen.  See the file "COPYING".  If not,
  *  write to:  The Free Software Foundation, Inc.,
- *             59 Temple Place - Suite 330,
- *             Boston,  MA  02111-1307, USA.
+ *             51 Franklin Street, Fifth Floor,
+ *             Boston, MA  02110-1301, USA.
  */
 
 tSCC zClientInput[] = "client-input";
@@ -63,6 +63,7 @@ inner_main( int argc, char** argv )
     initialize( argc, argv );
 
     procState = PROC_STATE_LOAD_DEFS;
+    ag_scmStrings_init();
     readDefines();
 
     /*
@@ -82,6 +83,7 @@ inner_main( int argc, char** argv )
     }
 
     unloadDefs();
+    ag_scmStrings_deinit();
 
     procState = PROC_STATE_DONE;
     exit( EXIT_SUCCESS );
@@ -101,9 +103,14 @@ main( int    argc,
 
     signalSetup( ignoreSignal, abendSignal );
 
+#if GUILE_VERSION >= 107000
+    if (getenv( "GUILE_WARN_DEPRECATED" ) == NULL)
+        putenv( "GUILE_WARN_DEPRECATED=no" );
+#endif
+
     gh_enter( argc, argv, inner_main );
     /* NOT REACHED */
-    return 0;
+    return EXIT_FAILURE;
 }
 
 static void
@@ -201,11 +208,14 @@ doneCheck( void )
     tSCC zErr[] =
         "Scheme evaluation error.  AutoGen ABEND-ing in template\n"
         "\t%s on line %d\n";
+#if GUILE_VERSION >= 107000
+    int exit_code = EXIT_SUCCESS;
+#endif
 
 #ifdef SHELL_ENABLED
     ag_scm_c_eval_string_from_file_line(
         "(if (> (string-length shell-cleanup) 0)"
-        "    (shell shell-cleanup) )", __FILE__, __LINE__ );
+        " (shell shell-cleanup) )", __FILE__, __LINE__ );
 #endif
 
     fflush( stdout );
@@ -238,6 +248,9 @@ doneCheck( void )
             pzOopsPrefix = zNil;
         }
 
+        if (OPT_VALUE_TRACE > TRACE_NOTHING)
+            scm_backtrace();
+
         fprintf( stderr, zErr, pCurTemplate->pzFileName, pCurMacro->lineNo );
 
         /*
@@ -250,6 +263,10 @@ doneCheck( void )
             closeOutput( AG_TRUE );
 #endif
         } while (pCurFp->pPrev != NULL);
+
+#if GUILE_VERSION >= 107000
+        exit_code = EXIT_FAILURE;
+#endif
         break; /* continue failure exit */
 
     default:
@@ -283,8 +300,13 @@ doneCheck( void )
     /*
      *  IF we diverted stderr, then now is the time to copy the text to stdout.
      */
-    if (pzTmpStderr == NULL)
+    if (pzTmpStderr == NULL) {
+#if GUILE_VERSION >= 107000
+        if (exit_code != EXIT_SUCCESS)
+            _exit( exit_code );
+#endif
         return;
+    }
 
     do {
         long pos = ftell( stderr );
@@ -306,6 +328,10 @@ doneCheck( void )
     unlink( pzTmpStderr );
     AGFREE( pzTmpStderr );
     pzTmpStderr = NULL;
+#if GUILE_VERSION >= 107000
+    if (exit_code != EXIT_SUCCESS)
+        _exit( exit_code );
+#endif
 }
 
 
