@@ -1,13 +1,13 @@
 
 /*
  *  expState.c
- *  $Id: expState.c,v 4.1 2005/01/01 00:12:33 bkorb Exp $
+ *  $Id: expState.c,v 4.8 2005/12/05 20:51:37 bkorb Exp $
  *  This module implements expression functions that
  *  query and get state information from AutoGen data.
  */
 
 /*
- *  AutoGen copyright 1992-2004 Bruce Korb
+ *  AutoGen copyright 1992-2005 Bruce Korb
  *
  *  AutoGen is free software.
  *  You may redistribute it and/or modify it under the terms of the
@@ -22,18 +22,27 @@
  *  You should have received a copy of the GNU General Public License
  *  along with AutoGen.  See the file "COPYING".  If not,
  *  write to:  The Free Software Foundation, Inc.,
- *             59 Temple Place - Suite 330,
- *             Boston,  MA  02111-1307, USA.
+ *             51 Franklin Street, Fifth Floor,
+ *             Boston, MA  02110-1301, USA.
  */
 
-STATIC int     entry_length(   char* pzName );
-STATIC int     count_entries(  char* pzName );
+/* = = = START-STATIC-FORWARD = = = */
+/* static forward declarations maintained by :mkfwd */
+static int
+entry_length( char* pzName );
+
+static int
+count_entries( char* pzName );
+
+static SCM
+find_entry_value( SCM op, SCM obj, SCM test );
+/* = = = END-STATIC-FORWARD = = = */
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
  *  EXPRESSION EVALUATION SUPPORT ROUTINES
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
-STATIC int
+static int
 entry_length( char* pzName )
 {
     tDefEntry**  papDefs = findEntryList( pzName );
@@ -55,7 +64,7 @@ entry_length( char* pzName )
 }
 
 
-STATIC int
+static int
 count_entries( char* pzName )
 {
     tDefEntry**  papDefs = findEntryList( pzName );
@@ -74,7 +83,7 @@ count_entries( char* pzName )
 }
 
 
-STATIC SCM
+static SCM
 find_entry_value( SCM op, SCM obj, SCM test )
 {
     ag_bool     isIndexed;
@@ -115,7 +124,7 @@ find_entry_value( SCM op, SCM obj, SCM test )
             return SCM_BOOL_F; /* Cannot match string -- not a text value */
         }
 
-        field  = gh_str02scm( pE->val.pzText );
+        field  = AG_SCM_STR02SCM( pE->val.pzText );
         result = gh_call2( op, field, test );
         if (! isIndexed)
             while (result == SCM_BOOL_F) {
@@ -124,7 +133,7 @@ find_entry_value( SCM op, SCM obj, SCM test )
                 if (pE == NULL)
                     break;
 
-                field = gh_str02scm( pE->val.pzText );
+                field = AG_SCM_STR02SCM( pE->val.pzText );
                 result = gh_call2( op, field, test );
             }
 
@@ -147,7 +156,7 @@ find_entry_value( SCM op, SCM obj, SCM test )
      */
     pzField[-1] = '.';
     {
-        SCM field   = gh_str02scm( pzField );
+        SCM field   = AG_SCM_STR02SCM( pzField );
         SCM result;
         tDefCtx ctx = currDefCtx;
 
@@ -186,7 +195,7 @@ find_entry_value( SCM op, SCM obj, SCM test )
 SCM
 ag_scm_base_name( void )
 {
-    return gh_str02scm( (char*)(void*)OPT_ARG( BASE_NAME ));
+    return AG_SCM_STR02SCM( (char*)(void*)OPT_ARG( BASE_NAME ));
 }
 
 
@@ -215,7 +224,7 @@ ag_scm_chdir( SCM dir )
         free( pCurDir );
     {
         char* pz = ag_scm2zchars( dir, zChdirDir );
-        pCurDir = malloc( SCM_LENGTH( dir ) + 1 );
+        pCurDir = malloc( AG_SCM_STRLEN( dir ) + 1 );
         strcpy( (char*)pCurDir, pz );
     }
     return dir;
@@ -239,7 +248,7 @@ ag_scm_count( SCM obj )
 {
     int ent_len = count_entries( ag_scm2zchars( obj, "ag object" ));
 
-    return gh_int2scm( ent_len );
+    return AG_SCM_INT2SCM( ent_len );
 }
 
 
@@ -254,7 +263,7 @@ ag_scm_count( SCM obj )
 SCM
 ag_scm_def_file( void )
 {
-    return gh_str02scm( (char*)(void*)pBaseCtx->pzFileName );
+    return AG_SCM_STR02SCM( (char*)(void*)pBaseCtx->pzCtxFname );
 }
 
 
@@ -337,8 +346,8 @@ ag_scm_ag_function_p( SCM obj )
 SCM
 ag_scm_match_value_p( SCM op, SCM obj, SCM test )
 {
-    if (  (! gh_procedure_p( op   ))
-       || (! gh_string_p(    obj  )) )
+    if (  (! AG_SCM_IS_PROC(  op  ))
+       || (! AG_SCM_STRING_P( obj )) )
         return SCM_UNDEFINED;
 
     if (OPT_VALUE_TRACE >= TRACE_EXPRESSIONS)
@@ -368,17 +377,17 @@ ag_scm_get( SCM agName, SCM altVal )
     tDefEntry*  pE;
     ag_bool     x;
 
-    if (! gh_string_p( agName ))
+    if (! AG_SCM_STRING_P( agName ))
          pE = NULL;
     else pE = findDefEntry( ag_scm2zchars( agName, "ag value" ), &x );
 
     if ((pE == NULL) || (pE->valType != VALTYP_TEXT)) {
-        if (gh_string_p( altVal ))
+        if (AG_SCM_STRING_P( altVal ))
             return altVal;
-        return gh_str02scm( "" );
+        return AG_SCM_STR02SCM( "" );
     }
 
-    return gh_str02scm( pE->val.pzText );
+    return AG_SCM_STR02SCM( pE->val.pzText );
 }
 
 
@@ -416,15 +425,15 @@ ag_scm_high_lim( SCM obj )
      *  ELSE search the twin list for the high entry
      */
     if (pE == NULL)
-        return gh_int2scm( 0 );
+        return AG_SCM_INT2SCM( 0 );
 
     if (isIndexed)
-        return gh_int2scm( pE->index );
+        return AG_SCM_INT2SCM( pE->index );
 
     if (pE->pEndTwin != NULL)
         pE = pE->pEndTwin;
 
-    return gh_int2scm( pE->index );
+    return AG_SCM_INT2SCM( pE->index );
 }
 
 
@@ -447,7 +456,7 @@ ag_scm_len( SCM obj )
 
     len = entry_length( ag_scm2zchars( obj, "ag value" ));
 
-    return gh_int2scm( len );
+    return AG_SCM_INT2SCM( len );
 }
 
 
@@ -473,9 +482,9 @@ ag_scm_low_lim( SCM obj )
      *  ELSE we have the low index.
      */
     if (pE == NULL)
-        return gh_int2scm( 0 );
+        return AG_SCM_INT2SCM( 0 );
 
-    return gh_int2scm( pE->index );
+    return AG_SCM_INT2SCM( pE->index );
 }
 
 
@@ -506,7 +515,7 @@ ag_scm_set_option( SCM opt )
 SCM
 ag_scm_suffix( void )
 {
-    return gh_str02scm( (char*)pzCurSfx );
+    return AG_SCM_STR02SCM( (char*)pzCurSfx );
 }
 
 
@@ -524,31 +533,37 @@ ag_scm_suffix( void )
 SCM
 ag_scm_tpl_file( SCM full )
 {
-    if (gh_boolean_p( full ) && SCM_NFALSEP( full )) {
+    if (AG_SCM_BOOL_P( full ) && SCM_NFALSEP( full )) {
         tSCC* sfx[] = { "tpl", NULL };
         char z[MAXPATHLEN];
         if (SUCCESSFUL( findFile( pzTemplFileName, z, sfx )))
-            return gh_str02scm( z );
+            return AG_SCM_STR02SCM( z );
     }
 
-    return gh_str02scm( (char*)(void*)pzTemplFileName );
+    return AG_SCM_STR02SCM( (char*)(void*)pzTemplFileName );
 }
 
 
 /*=gfunc tpl_file_line
  *
- * what:   get the template file and line number
+ * what:   get the template file+line number
  *
  * exparg: msg-fmt, formatting for line message, optional
  *
  * doc:
  *
- *  Returns the file and line number of the current template macro using either
- *  the default format, "from %s line %d", or else the format you supply.
- *  For example, if you want to insert a "C" language file-line directive,
- *  you would supply the format "# %2$d %1$s".  With the current version of
- *  AutoGen, it is even safe to use the formatting string, "%2$d".  AutoGen
- *  uses an argument vector version of printf:  @xref{snprintfv}.
+ *  Returns the file and line number of the current template macro using
+ *  either the default format, "from %s line %d", or else the format you
+ *  supply.  For example, if you want to insert a "C" language file-line
+ *  directive, you would supply the format "# %2$d \"%1$s\"", but that
+ *  is also already supplied with the scheme variable
+ *  @xref{SCM c-file-line-fmt}.  You may use it thus:
+ *  @example
+ *  (tpl-file-line c-file-line-fmt)
+ *  @end example
+ *
+ *  It is also safe to use the formatting string, "%2$d".  AutoGen uses
+ *  an argument vector version of printf: @xref{snprintfv}.
 =*/
 SCM
 ag_scm_tpl_file_line( SCM fmt )
@@ -558,7 +573,7 @@ ag_scm_tpl_file_line( SCM fmt )
     tCC*    pzFmt = zFmt;
     char*   pz;
 
-    if (gh_string_p( fmt ))
+    if (AG_SCM_STRING_P( fmt ))
         pzFmt = ag_scm2zchars( fmt, "file/line format" );
 
     {
@@ -572,18 +587,88 @@ ag_scm_tpl_file_line( SCM fmt )
     {
         void* args[2];
         args[0] = (void*)pCurTemplate->pzFileName;
-        args[1] = (void*)pCurMacro->lineNo;
+        args[1] = (void*)(long)pCurMacro->lineNo;
         sprintfv( pz, pzFmt, (snv_constpointer*)args  );
     }
 
     {
-        SCM res = gh_str02scm( pz );
+        SCM res = AG_SCM_STR02SCM( pz );
         if (pz != zScribble)
             AGFREE( (void*)pz );
 
         return res;
     }
 }
+
+
+/*=gfunc def_file_line
+ *
+ * what:   get a definition file+line number
+ *
+ * exparg: ag-name, name of AutoGen value
+ * exparg: msg-fmt, formatting for line message, optional
+ *
+ * doc:
+ *
+ *  Returns the file and line number of a AutoGen defined value, using
+ *  either the default format, "from %s line %d", or else the format you
+ *  supply.  For example, if you want to insert a "C" language file-line
+ *  directive, you would supply the format "# %2$d \"%1$s\"", but that
+ *  is also already supplied with the scheme variable
+ *  @xref{SCM c-file-line-fmt}.  You may use it thus:
+ *
+ *  @example
+ *  (def-file-line "ag-def-name" c-file-line-fmt)
+ *  @end example
+ *
+ *  It is also safe to use the formatting string, "%2$d".  AutoGen uses
+ *  an argument vector version of printf: @xref{snprintfv}.
+=*/
+SCM
+ag_scm_def_file_line( SCM obj, SCM fmt )
+{
+    tSCC    zFmt[]  = "from %s line %d";
+    tCC*    pzFmt   = zFmt;
+    char    zScribble[ 1024 ];
+    char*   pzScrib = zScribble;
+
+    tDefEntry*  pE;
+    ag_bool     x;
+
+    pE = findDefEntry( ag_scm2zchars( obj, "ag value" ), &x );
+
+    /*
+     *  IF we did not find the entry we are looking for
+     *  THEN return UNDEFINED
+     */
+    if (pE == NULL)
+        return SCM_UNDEFINED;
+
+    if (AG_SCM_STRING_P( fmt ))
+        pzFmt = ag_scm2zchars( fmt, "file/line format" );
+
+    {
+        size_t  maxlen = strlen( pE->pzSrcFile )
+                       + strlen( pzFmt ) + 4 * sizeof( int );
+        if (maxlen >= sizeof( zScribble ))
+            pzScrib = (char*)AGALOC( maxlen, "file-line buffer" );
+    }
+
+    {
+        void* args[2];
+        args[0] = (void*)pE->pzSrcFile;
+        args[1] = (void*)(long)pE->srcLineNum;
+        sprintfv( pzScrib, pzFmt, (snv_constpointer*)args  );
+    }
+
+    {
+        SCM res = AG_SCM_STR02SCM( pzScrib );
+        if (pzScrib != zScribble)
+            AGFREE( (void*)pzScrib );
+
+        return res;
+    }
+}
 /*
  * Local Variables:
  * mode: C
