@@ -1,8 +1,8 @@
 
 /*
- *  Time-stamp:      "2005-07-27 10:11:09 bkorb"
+ *  Time-stamp:      "2005-10-29 15:06:44 bkorb"
  *
- *  autoopts.h  $Id: autoopts.h,v 4.12 2005/07/27 17:26:32 bkorb Exp $
+ *  autoopts.h  $Id: autoopts.h,v 4.16 2005/11/07 00:22:05 bkorb Exp $
  *  Time-stamp:      "2005-02-14 05:59:50 bkorb"
  *
  *  This file defines all the global structures and special values
@@ -55,16 +55,7 @@
 #ifndef AUTOGEN_AUTOOPTS_H
 #define AUTOGEN_AUTOOPTS_H
 
-#ifndef   __EXTENSIONS__
-#  define __EXTENSIONS__
-#endif
-
-#include "config.h"
 #include "compat/compat.h"
-#include <sys/param.h>
-#include <sys/wait.h>
-#include <time.h>
-#include <utime.h>
 
 #define AO_NAME_LIMIT    127
 #define AO_NAME_SIZE     (AO_NAME_LIMIT + 1)
@@ -154,7 +145,7 @@ typedef enum {
 typedef struct {
     tOptDesc*  pOD;
     tCC*       pzOptArg;
-    tUL        flags;
+    tAoUL      flags;
     teOptType  optType;
 } tOptState;
 #define OPTSTATE_INITIALIZER(st) \
@@ -301,26 +292,6 @@ typedef struct {
 # endif
 #endif
 
-/*
- *  This is an output only structure used by text_mmap and text_munmap.
- *  Clients must not alter the contents and must provide it to both
- *  the text_mmap and text_munmap procedures.  BE ADVISED: if you are
- *  mapping the file with PROT_WRITE the NUL byte at the end MIGHT NOT
- *  BE WRITABLE.  In any event, that byte is not be written back
- *  to the source file.  ALSO: if "txt_data" is valid and "txt_errno"
- *  is not zero, then there *may* not be a terminating NUL.
- */
-typedef struct {
-    void*       txt_data;      /* text file data   */
-    size_t      txt_size;      /* actual file size */
-    int         txt_fd;        /* file descriptor  */
-    size_t      txt_full_size; /* mmaped mem size  */
-    int         txt_zero_fd;   /* fd for /dev/zero */
-    int         txt_errno;     /* warning code     */
-    int         txt_prot;      /* "prot" flags     */
-    int         txt_flags;     /* mapping type     */
-} tmap_info_t;
-
 /*
  *  Define and initialize all the user visible strings.
  *  We do not do translations.  If translations are to be done, then
